package org.openhab.binding.openlicht.handler;

import org.eclipse.jdt.annotation.NonNull;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingStatusDetail;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.io.transport.mqtt.MqttBrokerConnection;
import org.eclipse.smarthome.io.transport.mqtt.MqttBrokersObserver;
import org.eclipse.smarthome.io.transport.mqtt.MqttException;
import org.eclipse.smarthome.io.transport.mqtt.MqttMessageSubscriber;
import org.eclipse.smarthome.io.transport.mqtt.MqttService;
import org.openhab.binding.openlicht.internal.OpenLichtConfiguration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public abstract class AbstractMqttHandler extends BaseThingHandler
        implements MqttMessageSubscriber, MqttBrokersObserver {

    @NonNull
    protected final Logger logger = LoggerFactory.getLogger(AbstractMqttHandler.class);

    protected OpenLichtConfiguration config;

    private MqttService mqttService;

    private int usedTopicLength = 0;

    private MqttBrokerConnection currentBrokerConnection = null;

    public AbstractMqttHandler(Thing thing, MqttService mqttService) {
        super(thing);
        this.mqttService = mqttService;
    }

    @Override
    public void initialize() {
        config = getConfigAs(OpenLichtConfiguration.class);
        this.mqttService.addBrokersListener(this);
        updateStatus(ThingStatus.UNKNOWN, ThingStatusDetail.CONFIGURATION_PENDING, "Searching broker");
        for (MqttBrokerConnection broker : this.mqttService.getAllBrokerConnections()) {
            brokerAdded(broker);
        }
        if (currentBrokerConnection == null) {
            MqttBrokerConnection myBroker = this.mqttService.getBrokerConnection(config.brokerName);
            if (myBroker != null) {
                brokerAdded(myBroker);
            }
        }
        publish("init");
    }

    @Override
    public void brokerAdded(MqttBrokerConnection broker) {
        if (broker.getName().equals(config.brokerName)) {
            // this is our broker!
            currentBrokerConnection = broker;
            updateTopicLength();
            try {
                if (subscribeTopic()) {
                    broker.addConsumer(this);
                }
                updateStatus(ThingStatus.ONLINE);
                this.logger.info("Broker found, thing is online");
            } catch (MqttException e) {
                this.logger.error("Could not add subscriber to broker {}", config.brokerName);
                updateStatus(ThingStatus.OFFLINE, ThingStatusDetail.COMMUNICATION_ERROR,
                        "Could not add subscriber to broker " + config.brokerName);
            }
        } else {
            this.logger.debug("Got another broker, not interessted!");
        }
    }

    private void updateTopicLength() {
        // if subscribed to subtopics, a "/#" is added
        // then, we want only the part up to the last "/"
        usedTopicLength = getTopic().length() - (subscribeSubTopics() ? 1 : 0);
    }

    @Override
    public void brokerRemoved(MqttBrokerConnection broker) {
        if (broker.getName().equals(config.brokerName)) {
            // this was our broker!
            updateStatus(ThingStatus.OFFLINE, ThingStatusDetail.CONFIGURATION_PENDING, "Broker is offline");
            currentBrokerConnection = null;
        }
    }

    protected void publish(String message) {
        if (currentBrokerConnection == null) {
            logger.warn("Can't publish message, no connection");
            return;
        }
        String topic = subscribeSubTopics() ? (getTopic().substring(0, usedTopicLength) + "out")
                : (getTopic() + "/out");
        byte[] payload = message.getBytes();
        try {
            currentBrokerConnection.publish(topic, payload, null);
        } catch (MqttException e) {
            logger.error("Error while publishing", e);
        }
    }

    @Override
    public final String getTopic() {
        return config.topic + (getSubTopic() == null ? "" : "/" + getSubTopic()) + (subscribeSubTopics() ? "/#" : "");
    }

    protected final int getTopicLength() {
        return this.usedTopicLength;
    }

    /**
     * Subclasses may override this to subscribe to a special subtopic.
     * Do not add a leading not a trailing slash!
     *
     * @return <code>null</code> by default to not use subtopics
     */
    protected String getSubTopic() {
        return null;
    }

    /**
     * Subclasses may override this to prevent topic subscription
     *
     * @return <code>true</code> by default
     */
    protected boolean subscribeTopic() {
        return true;
    }

    /**
     * Subclasses may override this to also subscribe to any topic sharing the same prefix.
     * This means, that an additional "<code>/#</code>" is added at the end of the topic.
     *
     * @return <code>true</code> by default
     */
    protected boolean subscribeSubTopics() {
        return true;
    }

}
