/**
 * Copyright (c) 2014,2018 by the respective copyright holders.
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */
package org.openhab.binding.openlicht.internal;

import static org.openhab.binding.openlicht.internal.BindingConstants.*;

import java.util.Collections;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandlerFactory;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.thing.binding.ThingHandlerFactory;
import org.eclipse.smarthome.io.transport.mqtt.MqttService;
import org.openhab.binding.openlicht.handler.PolarHandler;
import org.openhab.binding.openlicht.handler.SkyWriterHATHandler;
import org.openhab.binding.openlicht.handler.SmartphoneHandler;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.slf4j.LoggerFactory;

/**
 * The {@link OpenLichtHandlerFactory} is responsible for creating things and thing
 * handlers.
 *
 * @author René Schöne - Initial contribution
 */
@NonNullByDefault
@Component(configurationPid = "binding.openlicht", service = ThingHandlerFactory.class)
public class OpenLichtHandlerFactory extends BaseThingHandlerFactory {

    private static final Set<ThingTypeUID> SUPPORTED_THING_TYPES_UIDS = Collections.unmodifiableSet(
            Stream.of(THING_TYPE_SKYWRITER_HAT, THING_TYPE_POLAR, THING_TYPE_SMARTPHONE).collect(Collectors.toSet()));
    private @Nullable MqttService service;

    @Override
    public boolean supportsThingType(ThingTypeUID thingTypeUID) {
        return SUPPORTED_THING_TYPES_UIDS.contains(thingTypeUID);
    }

    // @Override
    // protected void activate(ComponentContext componentContext) {
    // super.activate(componentContext);
    // };

    @Override
    protected @Nullable ThingHandler createHandler(Thing thing) {
        ThingTypeUID thingTypeUID = thing.getThingTypeUID();

        if (THING_TYPE_SKYWRITER_HAT.equals(thingTypeUID)) {
            return new SkyWriterHATHandler(thing, this.service);
        }
        if (THING_TYPE_POLAR.equals(thingTypeUID)) {
            return new PolarHandler(thing, this.service);
        }
        if (THING_TYPE_SMARTPHONE.equals(thingTypeUID)) {
            return new SmartphoneHandler(thing, this.service);
        }

        return null;
    }

    @Reference(cardinality = ReferenceCardinality.MANDATORY, policy = ReferencePolicy.STATIC)
    public void setMqttService(MqttService service) {
        LoggerFactory.getLogger(OpenLichtHandlerFactory.class).info("Setting mqtt service to {}", service);
        this.service = service;
    }

    public void unsetMqttService(MqttService service) {
        LoggerFactory.getLogger(OpenLichtHandlerFactory.class).info("Deleting mqtt service {}", service);
        this.service = null;
    }
}
